#!/etc/opt/openit/perl

BEGIN {
    push(@INC,"/opt/openit/lib" );
}


use strict;
use warnings;
use Getopt::Long;
use File::Copy qw(move);
use File::Basename;
use OpeniT::Config;
use OpeniT::DataTypes;


my $installationlog;
my $opt_debug;
my $opt_verbose;
my $opt_help;


unless ( GetOptions("help" => \ $opt_help,
         "debug"           => \ $opt_debug,
         "verbose"         => \ $opt_verbose
)) {
    &print_usage;
}

my $configdir = &get_configuration_dir();

# Update xml/conf files
if ($^O eq 'MSWin32' ){
	&_update_xml();
	&_update_confbuilder_conf( "$configdir/ConfBuilder-server.conf" );
}
else
{
	&_update_conf();
}
&_update_modules( "$configdir/modules" );
&_update_data_generation_config( "$configdir/data_generation_config" );
&_update_acc_types("$configdir/acc_types");
&_update_classvar("$configdir/classvar_mapping_config");





exit 0;

sub _update_conf
{
	&_update_daft_conf( "$configdir/daft.conf" ); 
}

sub _update_daft_conf
{

	my ($filter) = @_;
	&create_backup($filter);
	VERBOSE( "Updating $filter" );
	open(FILE, $filter) || die "Unable to read filter file '$filter'";
    my @lines = <FILE>;
	
	if ( grep {/dongle/} @lines )
	{
		VERBOSE( "daft is already updated" );
		return;
    }
	
	
	my $ind = 0;
	
	$ind++ while( $lines[$ind] !~ /openit2.DirName/ );
	$ind++;
	splice @lines, $ind, 0, "		dongle(DirName)=\"dongle\"{}\n";
	
	$ind++ while( $lines[$ind] !~ /filtrate/ );
	$ind++ while( $lines[$ind] !~ /openit2.String/ );
	$ind++;
	splice @lines, $ind, 0, "		dongle(String)=\"license-dongle-inventory \$BEG \$END P1D\"{}\n";
	
	$ind++ while( $lines[$ind] !~ /datagroups/ );
	$ind++ while( $lines[$ind] !~ /openit2.String/ );
	$ind++;
	splice @lines, $ind, 0, "		dongle(String)=\"PollParserWibuKey.license.wibukey,PollParserCodemtr.license.codemeter,PollParserVector.license.vector\"{}\n";
	$ind++;
	splice @lines, $ind, 0, "		pollcollector(String)=\"PollCollector.license.codemeter_dongle,PollCollector.license.wibukey_dongle,PollCollector.license.vector_dongle\"{}\n";
	
	_save_config( $filter, @lines );
	
}



sub _update_confbuilder_conf
{
	my ($filter) = @_;
	&create_backup($filter);
	VERBOSE( "Updating $filter" );
	open(FILE, $filter) || die "Unable to read filter file '$filter'";
    my @lines = <FILE>;

	
	if ( grep {/dongle/} @lines )
	{
		VERBOSE( "Confbuilder server conf is  is already updated" );
		return;
    }
	
	my $ind = $#lines;
	$ind++;
	
	my @insert = (
	"\n\"Components/filtrator/license-dongle-inventory.conf\"	{ \"Components/filtrator/license-dongle-inventory.xml\" }\n" );
	
	splice @lines, $ind, 0, @insert;
	
	_save_config( $filter, @lines );
}

sub _update_classvar
{
	my ($filter) = @_;
	&create_backup($filter);
	VERBOSE( "Updating $filter" );
	open(FILE, $filter) || die "Unable to read filter file '$filter'";
    my @lines = <FILE>;

	
	if ( grep {/Dongle/} @lines )
	{
		VERBOSE( "classvar_mapping_config is  is already updated" );
		return;
    }
	
	my $ind = $#lines;
	$ind-- while( $lines[$ind] !~ /\)/ );
	$ind-- while( $lines[$ind] !~ /\)\)/ );
	my $tmp = $lines[$ind] ;
	chomp $tmp;
	$lines[$ind] = "$tmp,";
	$ind++;
	
	splice @lines, $ind, 0, "\n";

	
	my @insert = (
	"\n  (Dongle^data^inventory,\n",
	"	(),\n",
	"  ((Dongle^type,Serial^number),( Vendor,Vendor^code ),(Feature, Feature^code),(Dongle^type,Vendor),(Dongle^type,Feature), (Dongle^type,Host^name) ))\n",
	"\n");

	
	splice @lines, $ind, 0, @insert;
	_save_config( $filter, @lines );
	
	
	
}

sub _update_xml()
{
	#update archiver.xml
	
	&_update_archiver_xml( "$configdir/Components/archiver.xml" ); 
	&_update_datagroup_generation( "$configdir/Components/datagroup-generation.xml" );
	
}

sub _update_data_generation_config
{
	my ($filter) = @_;
	&create_backup($filter);
	VERBOSE( "Updating $filter" );
	open(FILE, $filter) || die "Unable to read filter file '$filter'";
    my @lines = <FILE>;

	
	if ( grep {/dongle/} @lines )
	{
		VERBOSE( "data_generation_config is  is already updated" );
		return;
    }
	
	my $ind = $#lines;
	
	my @insert = (
	"\n\nTYPE:dongle:Dongle Data format:dynamic\n",
	"117:Dongle data inventory:license-dongle-inventory,P1D:on\n");
	
	splice @lines, $ind, 0, @insert;
	
	_save_config( $filter, @lines );
	
}


sub _update_acc_types
{
	my ($filter) = @_;
	&create_backup($filter);
	VERBOSE( "Updating $filter" );
	open(FILE, $filter) || die "Unable to read filter file '$filter'";
    my @lines = <FILE>;

	
	if ( grep {/Dongle/} @lines )
	{
		VERBOSE( "acc_types is already updated" );
		return;
    }
	
	
	my $ind = $#lines;
	# get last ')' 
	$ind-- while( $lines[$ind] !~ /\)/ );
	$ind--;
	$ind-- while( $lines[$ind] =~ /^\s*$/ );
	my $tmp = $lines[$ind] ;
	chomp $tmp;
	$lines[$ind] = "$tmp,";
	$ind++;
	
	splice @lines, $ind, 0, "\n";
	
	my @data_types =  list_data_type_ids();
	my $no_of_dt  =  $#data_types;
	DEBUG ("Number of datatypes: $no_of_dt");
	
	$no_of_dt++;
	my @dummy_id = ( $no_of_dt .. 116 );
	my @dummy_holders =() ;
	
	foreach my $dt ( @dummy_id )
	{
		my @dummy =(
			"\n",
			"  (Placeholder^for^datatype^$dt,\n",
			"   ((open)),\n",
			"   (open)),\n",
			"\n" 			
		);
		push (@dummy_holders , @dummy );
	}
	

	my @insert117 = 
		(
			" (Dongle^data^inventory,\n",
			"  (\n",
			"   (Count,Record^Count,x,SUM),\n",
			"   (Running^time,Accumulated^time^for^group^in^hours,Hour,SUM)\n",
			"  ),\n",
			"  (Dongle^type,\n",
			"   Serial^number,\n",
			"   Vendor,\n",
			"   Vendor^code,\n",
			"   Feature,\n",
			"   Feature^code,\n",
			"   Host^name)\n",
			"  ) \n"
	);
	my @insert = ( "\n" );
	push (@insert, @dummy_holders );
	push (@insert, @insert117 );
	
	#find last line
	$ind = $#lines;
	$ind-- while( $lines[$ind] !~ /\)/ );
	$ind--;
	
	
	splice @lines, $ind, 0, @insert;
	_save_config( $filter, @lines );
	
}

sub _update_modules
{
	my ($filter) = @_;
	VERBOSE( "Updating $filter" );
	open(FILE, $filter) || die "Unable to read filter file '$filter'";
    my @lines = <FILE>;

	
	if ( grep {/117/} @lines )
	{
		VERBOSE( "Modules is  is already updated" );
		return;
    }
	
	my $ind = 0;
	$ind++ while( $lines[$ind] !~ /License.Analyzer/ );
	$lines[$ind] = " (Open^iT^License^Analyzer, (9, 10, 16, 17, 37, 38, 43, 44, 45, 46, 47, 49, 50, 51, 52, 60, 61, 71, 72, 75, 76, 77, 83, 84, 85, 86, 89, 90, 91, 94, 95, 102, 104, 107, 108, 109, 110, 111, 112, 113, 114, 115, 116, 117))\n";
	
	&_save_config( $filter, @lines );
	
}

sub _update_datagroup_generation
{
	
	my ($filter) = @_;
	VERBOSE( "Updating $filter" );
	open(FILE, $filter) || die "Unable to read filter file '$filter'";
    my @lines = <FILE>;
    close(FILE);
	
	if ( grep {/dongle/} @lines )
	{
		VERBOSE( "datagroup-generation.xml filter is already updated" );
		return;
    }
	
	
	my $ind = 0;
	$ind++ while( $lines[$ind] !~ /openit2/ );
	$ind++ while( $lines[$ind] !~ /<\/Object>/ );
	$ind+=1;

	
	
	#insert Archive location
	my @insert = 
	(	"\n					<Object>\n",
		"						<Name>dongle</Name>\n",
		"						<Description>Data types for dongle inventory</Description>\n",
		"						<Value type=\"String\">PollParserWibuKey.license.wibukey,PollParserCodemtr.license.codemeter,PollParserVector.license.vector</Value>\n",
		"					</Object>\n"
	);
	
	splice @lines, $ind, 0, @insert;
	
	_save_config( $filter, @lines );
}

sub _update_archiver_xml
{
	
	my ($filter) = @_;
	VERBOSE( "Updating $filter" );

	open(FILE, $filter) || die "Unable to read filter file '$filter'";
    my @lines = <FILE>;
    close(FILE);
	
	if ( grep {/dongle/} @lines )
	{
		VERBOSE( "Archiver filter is already updated" );
		return;
    }
	
	
	my $ind = 0;
	$ind++ while( $lines[$ind] !~ /DirName.*openit2/ );
	$ind+=2;

	
	
	#insert Archive location
	my @insert = 
	(	"					<Object>\n",
		"						<Name>dongle</Name>\n",
		"						<Description>Supported type: Dongle data</Description>\n",
		"						<Value type=\"DirName\">dongle</Value>\n",
		"					</Object>\n"
	);
	
	splice @lines, $ind, 0, @insert;
	
	#insert Filtrate command
	@insert = ();
	
	$ind++ while( $lines[$ind] !~ /openit2/ );
	$ind++ while( $lines[$ind] !~ /type.openit2/ );
	$ind+=3;
	
	@insert = 
	(	"					<Object>\n",
		"						<Name>dongle</Name>\n",
		"						<Description>If type=dongle, run the following filter processes:</Description>\n",
		"						<Value type=\"String\">license-dongle-inventory \$BEG \$END P1D</Value>\n",
		"					</Object>\n"
	);
	
	splice @lines, $ind, 0, @insert;

	_save_config( $filter, @lines );
}

sub _save_config
{
	
	my ( $cfg, @lines ) = @_; 
	#&create_backup($cfg);
	open(OUT, ">$cfg") || die "Unable to overwrite filter file '$cfg'";
    print OUT @lines;
    close(OUT);
}

sub create_backup 
{
	my ($filter) = @_;
	DEBUG ( "Creating $filter backup " );
	my $new = "$filter.new";
    my $old = "$filter.old";

    open(FILE, $filter) || die "Unable to read filter file '$filter'";
    my @lines = <FILE>;
    close(FILE);
	
	open(OUT, ">$old") || die "Unable to create backup file '$old'";
    print OUT @lines;
    close(OUT);
}

sub get_configuration_dir()
{
	my $configdir;
	if ($^O eq 'MSWin32' )
	{
		my $cmd = `openit_freemould --script installation_info.oms`;
		chomp $cmd;
	
		$configdir =$cmd . "Configuration"
	}
	else
	{
		$configdir = `. /tmp/openit-cfg-shenv && echo \$CONFIG_DIR`;
	}
	chomp $configdir;
	DEBUG ("Got Confidir '$configdir'");
	return $configdir;
}


sub DEBUG 
{
	my ($log) = @_;
	$installationlog .= $log;
	if ( $opt_debug )
	{
		print ( "DEBUG: $log\n" );
	}
}

sub VERBOSE 
{
	my ($log) = @_;
	$installationlog .= $log;
	if ( $opt_verbose )
	{
		print ( "-- $log\n" );
	}
}

sub NOTICE 
{
	my ($log) = @_;
	$installationlog .= $log;
	print ( "NOTICE: $log\n" );
	
}
